// Instruments v0.1 by BlahGoD

using System;
using Server;

namespace Server.Scripts.Items.Instruments
{
	public abstract class BaseInstrument : Item
	{
		private int m_WellSound, m_BadlySound;

		public BaseInstrument( int itemID, int wellSound, int badlySound ) : base( itemID )
		{
			m_WellSound = wellSound;
			m_BadlySound = badlySound;
		}

		public BaseInstrument( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( m_WellSound );
			writer.Write( m_BadlySound );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_WellSound = reader.ReadInt();
					m_BadlySound = reader.ReadInt();

					break;
				}
			}
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( from.CheckSkill( SkillName.Musicianship, 0.0, 100.0 ) )
			{
				PlayInstrumentWell( from );
			} 
			else 
			{
				PlayInstrumentBadly( from );
			}
		}

		public void PlayInstrumentWell( Mobile from )
		{
			from.PlaySound( m_WellSound );
		}

		public void PlayInstrumentBadly( Mobile from )
		{
			from.PlaySound( m_BadlySound );
		}
	}
}