using System;
using System.Collections;
using Server;
using Server.Scripts.Items;
using Server.Scripts.Items.Doors;

namespace Server.Scripts.Multis
{
	public abstract class BaseHouse : Item
	{
		private ArrayList m_Doors;
		private ArrayList m_Components;
		private ArrayList m_CoOwners;
		private ArrayList m_Friends;
		private HouseSign m_Sign;
		private Mobile m_Owner;

		public BaseHouse( int multiID ) : base( multiID | 0x4000 )
		{
			m_Doors = new ArrayList();
			m_Components = new ArrayList();
			m_CoOwners = new ArrayList();
			m_Friends = new ArrayList();
		}

		public BaseHouse( Serial serial ) : base( serial )
		{
		}

		public override void OnMapChange()
		{
			if ( m_Components == null )
				return;

			foreach ( Item item in m_Components )
			{
				item.Map = this.Map;
			}
		}

		public override void OnLocationChange( Point3D oldLocation )
		{
			if ( m_Components == null )
				return;

			int x = base.Location.X - oldLocation.X;
			int y = base.Location.Y - oldLocation.Y;
			int z = base.Location.Z - oldLocation.Z;

			foreach ( Item item in m_Components )
			{
				item.Location = new Point3D( item.X + x, item.Y + y, item.Z + z );
			}
		}

		public void AddDoor( BaseDoor door, Point3D offset )
		{
			door.MoveToWorld( offset, this.Map );

			m_Doors.Add( door );
			m_Components.Add( door );
		}

		public void AddComponent( Item item, Point3D offset )
		{
			item.MoveToWorld( offset, this.Map );

			m_Components.Add( item );
		}

		public void SetSign( Point3D offset )
		{
			m_Sign = new HouseSign( this );

			m_Sign.MoveToWorld( offset, this.Map );

			m_Components.Add( m_Sign );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( m_Owner );

			WriteMobileList( writer, m_CoOwners );
			WriteMobileList( writer, m_Friends );

			WriteItemList( writer, m_Doors );
			WriteItemList( writer, m_Components );

			writer.Write( m_Sign );
		}

		private void WriteMobileList( GenericWriter writer, ArrayList list )
		{
			writer.Write( list.Count );

			for ( int i = 0; i < list.Count; ++i )
				writer.Write( (Mobile)list[i] );
		}

		private ArrayList ReadMobileList( GenericReader reader )
		{
			int count = reader.ReadInt();

			ArrayList list = new ArrayList( count );

			for ( int i = 0; i < count; ++i )
			{
				Mobile m = reader.ReadMobile();

				if ( m != null )
					list.Add( m );
			}

			return list;
		}

		private void WriteItemList( GenericWriter writer, ArrayList list )
		{
			writer.Write( list.Count );

			for ( int i = 0; i < list.Count; ++i )
				writer.Write( (Item)list[i] );
		}

		private ArrayList ReadItemList( GenericReader reader )
		{
			int count = reader.ReadInt();

			ArrayList list = new ArrayList( count );

			for ( int i = 0; i < count; ++i )
			{
				Item item = reader.ReadItem();

				if ( item != null )
					list.Add( item );
			}

			return list;
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Owner = reader.ReadMobile();

					m_CoOwners = ReadMobileList( reader );
					m_Friends = ReadMobileList( reader );

					m_Doors = ReadItemList( reader );
					m_Components = ReadItemList( reader );

					m_Sign = reader.ReadItem() as HouseSign;

					break;
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile Owner
		{
			get
			{
				return m_Owner;
			}
			set
			{
				m_Owner = value;
			}
		}

		public ArrayList CoOwners
		{
			get
			{
				return m_CoOwners;
			}
		}

		public ArrayList Friends
		{
			get
			{
				return m_Friends;
			}
		}

		public ArrayList Doors
		{
			get
			{
				return m_Doors;
			}
		}

		public ArrayList Components
		{
			get
			{
				return m_Components;
			}
		}

		public HouseSign Sign
		{
			get
			{
				return m_Sign;
			}
			set
			{
				m_Sign = value;
			}
		}

		public abstract Point3D EjectOffset{ get; }
	}
}