using System;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.Third
{
	public class PoisonSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Poison", "In Nox",
				SpellCircle.Third,
				203,
				Reagent.Nightshade,
				Reagent.SpidersSilk
			);

		public PoisonSpell( Mobile caster, Server.Items.SpellScroll scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( Mobile m )
		{
			if ( !Caster.CanSee( m ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( Caster.HarmfulCheck( m ) && CheckSequence() )
			{
				if ( CheckResisted( m ) )
				{
					m.SendLocalizedMessage( 501783 ); // You feel yourself resisting magical energy.

					if ( m.Spell != null && m.Spell.State == SpellState.Casting )
					{
						m.Spell.Disturb();
					}

					m.Paralyzed = false;
				}
				else
				{
					double skillTotal = (Caster.Skills[SkillName.Magery].Value + Caster.Skills[SkillName.Poisoning].Value) / 2.0;
					Poison p;

					if ( skillTotal >= 100.0 && (Utility.RandomDouble() <= 0.1) ) // 10% chance at 100.0 to go deadly
					{
						p = Poison.Deadly;
					}
					else if ( skillTotal >= 85.0 )
					{
						p = Poison.Greater;
					}
					else if ( skillTotal >= 65.0 )
					{
						p = Poison.Regular;
					}
					else
					{
						p = Poison.Lesser;
					}

					m.Poison = p;
				}

				if ( Caster != m )
				{
					Caster.Direction = Caster.GetDirectionTo( m );
				}

				Caster.PlaySound( 0x205 );
				Effects.SendTargetEffect( m, 0x374A, 16 );
			}

			FinishSequence();
		}

		private class InternalTarget : Target
		{
			private PoisonSpell m_Owner;

			public InternalTarget( PoisonSpell owner ) : base( 12, false, TargetFlags.Harmful )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is Mobile )
				{
					m_Owner.Target( (Mobile)o );
				}
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}